/*********************************************************
	Project Details							
**********************************************************

IQS5xx i2c MASTER example code

Compatible with IQS5xx version: 40_0_55

Azoteq

--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
*/


#include <p18cxxx.h>
#include "includes.h"

/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
							----    init()    ----
	Description: 	Responsible for initialising the PIC18F4550 ports and calling other 
				initialisation functions.
				Note that the CAL flag has to be clear before settings can be sent to the IQS5xx,
				else the calibration process will be interrupted.

*/
void init(void)
{
	ADCON1 = 0x0F;								//PORTA all digital operation	
	LATA = 0x02;
	TRISA = 0x02;								//RA3 is power output for IQS5xx, RA1 (RDY) input.
	TRISD = 0x00; 								//configure PORTD for output
	LATD = 0x0F;								//LEDS off

	LATB = 0xFF;
	TRISB = 0xFF;								//PORTB for input
	INTCON2bits.RBPU = 0;						//For PIC18LF4550-Eval-Rev4A hardware with 74HC573 latch
	TRISCbits.TRISC0 = 0;
	LATCbits.LATC0  = 1;						//un-latch 74HC573 to make OUTD follows LATD
	TRISCbits.TRISC1 = 0;						//enable the latch
	LATCbits.LATC1 = 0;

	Comms_init();
	

	LATB = 0xFF;
	LATD = 0x0F;

	button_mem = PORTB;
	InitGraph();

/*
	Place other functions responsible for hardware initialisation here.
*/	
	IQS5xx_Settings();
}

/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	Description:	Initialises the SPI module on the PIC18F4550	

*/
void Comms_init()							//Initiates i2c module on PIC18F4550
{
	SSPADD = 0x02;							// settings for I2C frequency - 416kHz
	SSPSTAT |= 0x80;						// slew rate control for high speed (400kHz)
	// SSPADD = 0x08;							// settings for I2C frequency - 138kHz
		
	TRISB = TRISB | 0x03;					//set TRISB<0> Make I2c SDA and SCL inputs
	
	PIR1bits.SSPIF = 0;
	SSPCON1 = 0x28;							//enables i2c, set to master 
	
	LATA = LATA | 0x08;						//switch on the IQS5xx
}

/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
						----    CommsIQS5xx_send()    ----
	Description:	Transmits the data byte given as parameter via the I2C of the PIC18F4550
				Note that the I2C communication channel must already be active before calling this function,
				as no start bits are included in this function
				
	Parameters:		send_data - data byte to be transmitted via the I2C

*/
void CommsIQS5xx_send(unsigned char send_data)
{
	SSPBUF = send_data;						// write transmit byte to buffer
	while (PIR1bits.SSPIF == 0)				// wait for transmit complete flag
	{}
	PIR1bits.SSPIF = 0;						//clear flag	

	while (SSPCON2bits.ACKSTAT == 1)		//verify IQS5xx acknowledge
	{}

}

/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
						----    CommsIQS5xx_read_ack()    ----
	Description:	Enables the Master Receive Mode of the I2C module on the PIC18F4550. The data received is
				returned and an (ACK) acknowledge is sent to the IQS5xx to indicate that more byte read's will
				still take place.
				
	Return:		The data received via the I2C

*/

unsigned char CommsIQS5xx_read_ack(void)
{
	unsigned char temp;
	
	SSPCON2bits.RCEN = 1;					// enable master receiver mode
	while (PIR1bits.SSPIF == 0)				// wait for byte received flag
	{}
	PIR1bits.SSPIF = 0;						//clear flag

	while (SSPSTATbits.BF == 0)				// wait for buffer full flag (receive complete)			
	{}
	temp = SSPBUF;							// store received byte

	SSPCON2bits.ACKDT = 0;					// enable ACK			
	SSPCON2bits.ACKEN = 1;					// execute ACK sequence

	while (PIR1bits.SSPIF == 0)				// Wait for ACK transmission complete
	{}
	PIR1bits.SSPIF = 0;						//clear flag
	
	while (SSPCON2bits.ACKEN == 1) 			//verify acknowledge sequence is complete
	{}

	return temp;
}

/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
						----    CommsIQS5xx_read_nack()    ----
	Description:	Enables the Master Receive Mode of the I2C module on the PIC18F4550. The data received is
				returned and a NACK acknowledge is sent to the IQS5xx to indicate that this was the final read
				of the current continuous read block. 
				
	Return:		The data received via the I2C

*/
unsigned char CommsIQS5xx_read_nack(void)
{
	unsigned char temp;
	
	SSPCON2bits.RCEN = 1;					// enable master receiver mode
	while (PIR1bits.SSPIF == 0)             // wait for byte received flag
	{}                                      
	PIR1bits.SSPIF = 0;			            //clear flag
                                            
	while (SSPSTATbits.BF == 0)             // wait for buffer full flag (receive complete
	{}                                      
	temp = SSPBUF;                          // store received byte
                                            
	SSPCON2bits.ACKDT = 1;                  // enable NACK			
	SSPCON2bits.ACKEN = 1;                  // execute NACK sequence
                                            
	while (PIR1bits.SSPIF == 0)             // Wait for NACK transmission complete
	{}                                      
	PIR1bits.SSPIF = 0;		                //clear flag
	                                        
	while (SSPCON2bits.ACKEN == 1) 			//verify acknowledge sequence is complete
	{}

	return temp;
}

/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
						----    CommsIQS5xx_start()    ----
	Description:	This function waits for the RDY to become high, indicating the start of a communication
				window with the IQS5xx.
				An i2c START condition is then generated.

*/

void CommsIQS5xx_start(void)
{
	while (PORTAbits.RA1 == 0)		//wait for ready
	{}

	SSPCON2bits.SEN = 1;			//start condition

	while (PIR1bits.SSPIF == 0)		//wait for start condition to be generated
	{}
	PIR1bits.SSPIF = 0;				//clear flag

	while (SSPCON2bits.SEN == 1)	// verify start is complete
	{}
}

/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
						----    CommsIQS5xx_repeat_start()    ----
	Description:	An i2c START condition is generated.

*/

void CommsIQS5xx_repeat_start(void)
{
	SSPCON2bits.RSEN = 1;			//start condition

	while (PIR1bits.SSPIF == 0)		//wait for start condition to be generated
	{}
	PIR1bits.SSPIF = 0;				//clear flag

	while (SSPCON2bits.RSEN == 1)	// verify start is complete
	{}
}

/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
						----    CommsIQS5xx_stop()    ----
	Description:	An i2c STOP condition is generated.

*/

void CommsIQS5xx_stop(void)
{
	SSPCON2bits.PEN = 1;			//stop condition

	while (PIR1bits.SSPIF == 0)		//wait for stop condition to be generated
	{}
	PIR1bits.SSPIF = 0;				//clear flag

	while (SSPCON2bits.PEN == 1)	// verify stop is complete
	{}
}